@@ -283,15 +283,12 @@ static semanage_list_t *get_home_dirs(genhomedircon_settings_t * s)
 	char *rbuf = NULL;
 	char *path = NULL;
 	long rbuflen;
-	uid_t temp, minuid = 0;
+	uid_t temp, minuid = 500;
 	int minuid_set = 0;
 	struct passwd pwstorage, *pwbuf;
 	struct stat buf;
 	int retval;
 
-	shells = get_shell_list();
-	assert(shells);
-
 	path = semanage_findval(PATH_ETC_USERADD, "HOME", "=");
 	if (path && *path) {
 		if (semanage_list_push(&homedir_list, path))
@@ -324,13 +321,14 @@ static semanage_list_t *get_home_dirs(genhomedircon_settings_t * s)
 	if (!(s->usepasswd))
 		return homedir_list;
 
+	shells = get_shell_list();
+	assert(shells);
+
 	path = semanage_findval(PATH_ETC_LOGIN_DEFS, "UID_MIN", NULL);
 	if (path && *path) {
 		temp = atoi(path);
-		if (!minuid_set || temp < minuid) {
-			minuid = temp;
-			minuid_set = 1;
-		}
+		minuid = temp;
+		minuid_set = 1;
 	}
 	free(path);
 	path = NULL;
@@ -346,11 +344,6 @@ static semanage_list_t *get_home_dirs(genhomedircon_settings_t * s)
 	free(path);
 	path = NULL;
 
-	if (!minuid_set) {
-		minuid = 500;
-		minuid_set = 1;
-	}
-
 	rbuflen = sysconf(_SC_GETPW_R_SIZE_MAX);
 	if (rbuflen <= 0)
 		goto fail;
@@ -406,11 +399,13 @@ static semanage_list_t *get_home_dirs(genhomedircon_settings_t * s)
 		WARN(s->h_semanage, "Error while fetching users.  "
 		     "Returning list so far.");
 	}
+
+	if (semanage_list_sort(&homedir_list))
+		goto fail;
+
 	endpwent();
 	free(rbuf);
 	semanage_list_destroy(&shells);
-	if (semanage_list_sort(&homedir_list))
-		goto fail;
 
 	return homedir_list;
 
@@ -772,12 +767,14 @@ static int setup_fallback_user(genhomedircon_settings_t * s)
 			if (semanage_user_query(s->h_semanage, key, &u) < 0)
 			{
 				prefix = name;
-				level = "s0";
+				level = FALLBACK_USER_LEVEL;
 			}
 			else
 			{
 				prefix = semanage_user_get_prefix(u);
 				level = semanage_user_get_mlslevel(u);
+				if (!level)
+					level = FALLBACK_USER_LEVEL;
 			}
 
 			if (set_fallback_user(s, seuname, prefix, level) != 0)
@@ -861,9 +858,11 @@ static genhomedircon_user_entry_t *get_users(genhomedircon_settings_t * s,
 		if (u) {
 			prefix = semanage_user_get_prefix(*u);
 			level = semanage_user_get_mlslevel(*u);
+			if (!level)
+				level = FALLBACK_USER_LEVEL;
 		} else {
 			prefix = name;
-			level = "s0";
+			level = FALLBACK_USER_LEVEL;
 		}
 
 		retval = getpwnam_r(name, &pwstorage, rbuf, rbuflen, &pwent);
@@ -935,16 +934,21 @@ static int write_gen_home_dir_context(genhomedircon_settings_t * s, FILE * out,
 		if (write_home_dir_context(s, out, homedir_context_tpl,
 					   users->name,
 					   users->sename, users->home,
-					   users->prefix, users->level)) {
-			return STATUS_ERR;
-		}
+					   users->prefix, users->level))
+			goto err;
 		if (write_user_context(s, out, user_context_tpl, users->name,
-				       users->sename, users->prefix)) {
-			return STATUS_ERR;
-		}
+				       users->sename, users->prefix))
+			goto err;
 	}
 
 	return STATUS_SUCCESS;
+err:
+	for (; users; pop_user_entry(&users)) {
+	/* the pop function takes care of all the cleanup
+	 * so the loop body is just empty */
+	}
+
+	return STATUS_ERR;
 }
 
 /**
